;------------------------------------------------------------------
; Rotary display/clock
;------------------------------------------------------------------
	processor 16F876
	include	<16F876~1.INC>
	__config B'11110100110010'
	radix	dec
	errorlevel 0,-302	;turn off bank switch warnings
Xtal	equ	14745600	;Crystal frequency

	include	<VARIABLE.INC>	;pinout and ram registers
	include	<MACRO.INC>	;text line macro
;------------------------------------------------------------------
; PROGRAM START ORIGIN
;------------------------------------------------------------------
	org	H'0000'		;start at PCL 0000
	nop
	call	init
	goto	start
	nop
;------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE (INT)
;------------------------------------------------------------------
	org	H'0004'
irq	movwf	w_temp		;save W and STATUS registers
	swapf	STATUS,W
	movwf	s_temp
	movf	PCLATH,W	;save memory page
	movwf	p_temp
	clrf	PCLATH		;memory page 0
	bsf	RP0		;bank1
	bcf	D0		;set D0 as output to enable bus
	bcf	RP0		;bank0
	btfsc	PHASE		;phase 0 or 1?
	goto	pr_dgtl		;phase 1, do digital
	call	analog		;phase 0, do analog
	bsf	PHASE
	goto	ret_int
pr_dgtl	call	digital		;display digital time/date
	call	disp_lw		;lower half of display
	bcf	PHASE
ret_int	bsf	RP0		;bank1
	bsf	D0		;set D0 as input to receive next frame
	bcf	RP0		;bank0
c_index	btfsc	ZERO		;latch = 0?
	goto	l_index		;no, continue
	btfss	INDEX		;yes, incoming index signal?
	goto	l_index		;no, continue
	bcf	PHASE		;sync phase
	clrf	dot		;latch=0 & index=1, reset dot counter
	movlw	1
	movwf	hourdot		;sync hour dots
	bsf	ZERO		;set latch to prevent multiple reset
l_index	btfss	INDEX		;index = 1?
	bcf	ZERO		;no, index=0, reset latch
	movf	p_temp,W	;recall memory page
	movwf	PCLATH
	swapf	s_temp,W	;recall STATUS register
	movwf	STATUS
	swapf	w_temp,F	;restore W without affecting status bits
	swapf	w_temp,W
	bcf	INTF		;clear interrupt source flag
	retfie			;return from interrupt
;------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------
start	call	alloff		;all outputs off
	call	ledtest		;do ledtest
	call	adjust		;put out adjusting frequency
	call	settime		;set the time
	call	alloff		;clear leds
	call	bcdtime		;translate BCD time setting
	call	buffer_time	;initialize time buffer
	call	buffer_date	;initialize date/month/year buffer
;	call	ledtest		;wait a moment
;	bsf	MOTOR		;soft start motor
	bsf	RP0		;bank1
	bsf	D0		;set D0 as input to receive first frame
	bcf	RP0		;bank0
	bcf	INTF		;clear interrupt source bit
	bsf	GIE		;start display scan
main	call	wait		;keep time
	incf	s_hand,F	;update analog seconds
	decfsz	pre2,F
	goto	main
	movlw	4
	movwf	pre2
	call	keep_time	;keep digital time (1x p/s)
	call	buffer_time	;update time text buffer
	btfsc	DAY		;day has passed?
	call	buffer_date	;prepare date text buffer
	bcf	DAY
	goto	main
;------------------------------------------------------------------
; 0-phase: update analog clock (every .25s)
;------------------------------------------------------------------
analog	btfsc	SCRL
	call	rectangle
an_ct	decfsz	hourdot,F	;update counter
	goto	dsphand
	bsf	YL1		;set dot
	movlw	20		;reload counter
	movwf	hourdot
dsphand	movlw	255
	movwf	PORTB
	movf	dot,W
	subwf	s_hand,W	;reached position of seconds hand?
	btfss	Z
	goto	dspm		;no
	movlw	15		;yes
	goto	dsph
dspm	movf	dot,W
	subwf	m_hand,W	;reached position of minutes hand?
	btfss	Z
	goto	dsps		;no
	movlw	14		;yes
	goto	dsph
dsps	movf	dot,W
	subwf	h_hand,W	;reached position of hours hand?
	btfss	Z
	return			;no
	movlw	12
dsph	bsf	YL2
	bsf	YL3
	movwf	PORTA
	clrf	PORTA
	return
;------------------------------------------------------------------
; phase 1: display digital time and date
;------------------------------------------------------------------
digital	call	wrbus		;update buffers
	bcf	YL1		;clear hour dot
	incf	dot,F		;increase dot counter
update	movlw	high chars
	movwf	PCLATH
	digit	0,time_start,8,l0cnt,l0tbuf,l0scan,l0pbuf
	digit	1,day_start,9,l1cnt,l1tbuf,l1scan,l1pbuf
	digit	2,date_start,12,l2cnt,l2tbuf,l2scan,l2pbuf
	digit	3,year_start,4,l3cnt,l3tbuf,l3scan,l3pbuf
	return
;------------------------------------------------------------------
; logo or scroller
;------------------------------------------------------------------
disp_lw	btfsc	SCRL		;scroller active?
	goto	disp_sc		;yes then jump there
	movf	dot,W
	subwf	graph_start,W	;reached starting position?
	btfsc	Z
	clrf	gscan		;yes, reset scanline counter
	movf	gscan,W		;get scanline counter
	sublw	64		;last scanline?
	btfsc	Z		;no, next scanline
	return			;yes, ready
	movlw	high logo0
	movwf	PCLATH		;point to the right page
	movf	gmem,W		;get data pointer
	call	logo_tb		;get data from table
	movwf	l3pbuf		;put in port buffer
	incf	gmem,F
	movf	gmem,W
	call	logo_tb
	movwf	l2pbuf
	incf	gmem,F
	movf	gmem,W
	call	logo_tb
	movwf	l1pbuf
	incf	gmem,F
	movf	gmem,W
	call	logo_tb
	movwf	l0pbuf
	incf	gmem,F
	incf	gscan,F
	return
;------------------------------------------------------------------
; scrolling message
;------------------------------------------------------------------
disp_sc	movf	dot,W
	sublw	65		;just before start?
	btfsc	Z
	goto	update_scroller
	movf	dot,W
	sublw	72		;starting frame?
	btfsc	Z
	clrf	sscan		;yes, clear scanline counter
	movf	sscan,W		;get scanline counter
	sublw	96		;last scanline?
	btfsc	Z
	return			;yes, end
disp_ct	movf	scroll,W	;get scroll position counter
	addwf	sscan,W		;add scanline counter
	movwf	e_temp		;put into temp
	addlw	-96		;wrap around?
	btfss	C
	movf	e_temp,W	;no, put smaller value into pointer
	addlw	H'10'
	movwf	FSR
	bsf	IRP		;set indirect bank 2/3
	movf	INDF,W		;get data from pointer
	bcf	IRP		;reset bank bit
	movwf	l1pbuf		;put into line 1 port buffer
	incf	sscan,F
	return
;------------------------------------------------------------------
; draw rectangle around scroller (called from phase 0)
;------------------------------------------------------------------
rectangle
rec_1	movf	dot,W
	sublw	68		;frame 68
	btfsc	Z
	goto	rec_1b
	movf	dot,W
	sublw	170		;frame 170
	btfss	Z
	goto	rec_2
rec_1b	movlw	B'00000111'	;open/close rectangle
	movwf	l2pbuf
	movlw	B'11111111'
	movwf	l1pbuf
	movlw	B'11110000'
	movwf	l0pbuf	
	return
rec_2	movf	dot,W
	sublw	70		;frame 70
	btfss	Z
	goto	rec_3
	movlw	B'00000110'	;midpiece rectangle
	movwf	l2pbuf
	movlw	B'00000000'
	movwf	l1pbuf
	movlw	B'00110000'
	movwf	l0pbuf
	return
rec_3	movf	dot,W
	sublw	168		;frame 168
	btfsc	Z
	goto	rec_4
	movf	dot,W
	sublw	172		;frame 172: end rectangle
	btfss	Z
	return
	clrf	l2pbuf
	clrf	l0pbuf
rec_4	clrf	l1pbuf		;trail scroller with 0s
	return
;------------------------------------------------------------------
; INITIALIZATION
;------------------------------------------------------------------
init	bsf	RP0		;bank 1
	movlw	B'11100000'	;set port A
	movwf	TRISA
	movlw	B'00000000'	;set port B
	movwf	TRISB
	movlw	B'00110000'	;set port C
	movwf	TRISC
	movlw	B'11111111'	;set timer and pull-ups
	movwf	OPTION_REG
	bsf	INTE		;set interrupt source
	movlw	B'00000110'	;disable AD converter
	movwf	ADCON1
	bcf	RP0		;bank 0
	clrf	PORTA
	clrf	PORTB
	clrf	PORTC
	movlw	B'01111111'	;timer 2 maximum pre- and postscaler
	movwf	T2CON
	movlw	H'20'
	call	clrmem		;clear RAM page 0 (20-7F)
	movlw	H'10'
	bsf	IRP
	call	clrmem		;clear RAM page 2 (110-17F)
	bcf	IRP
	movlw	15		;preload prescalers 1 and 2
	movwf	pre1
	movlw	4
	movwf	pre2
	movlw	27
	movwf	l0tbuf+2	;colons in time display
	movwf	l0tbuf+5
	movlw	32
	movwf	l3tbuf		;"200x"
	movlw	30
	movwf	l3tbuf+1
	movwf	l3tbuf+2
	movlw	223
	movwf	time_start	;starting frame of time display
	movlw	210
	movwf	day_start
	movlw	232
	movwf	date_start
	movlw	230
	movwf	year_start
	movlw	88
	movwf	graph_start	;starting frame of logo display
	return
;------------------------------------------------------------------
; inclusions
;------------------------------------------------------------------
	include	<routines.inc>
	include	<charset.inc>
	include <graphics.inc>
;------------------------------------------------------------------
	end
;------------------------------------------------------------------
